#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  4 10:35:32 2019

@author: martindrech
"""

from PyQt5.QtWidgets import QMainWindow, QPushButton, QLabel
from PyQt5 import QtCore, QtGui
from pyqtgraph import dockarea
import pyqtgraph as pg
from pyqtgraph import console

from subclasses.pyqtgraph_subclasses import CustomRectangularROI
from zylaCameraWorker import CameraWorker
from subclasses.cameraParameterTree import CameraParameterTree
from subclasses.controllayouts import LaserControlLayout, LaserScanLayout



#from drivers.andorzyla import AndorZyla
from drivers.dummyAndor import AndorZyla
from measurement import MeasurementFrame
import sys
import numpy as np


class CameraGuiMainWindow(QMainWindow):
    
    signalWithMeasurementParameters = QtCore.pyqtSignal(object)
    
    def __init__(self):
        super().__init__()

        # self.__init__

        self.counter = 0
        self.frames_checker_timer = QtCore.QTimer()

        self.initUI()

    def initUI(self):

        self.win = QtGui.QMainWindow()
        self.win.setWindowTitle("Toy GUI")
        self.win.resize(1000, 500)

        self.area = dockarea.DockArea()
        self.win.setCentralWidget(self.area)
        self.cw = QtGui.QWidget()
        self.win.show()

        self.dCameraView = dockarea.Dock("Zyla Camera View")
        self.dCameraTimeAnal = dockarea.Dock("Zyla Camera Time Analysis")
        self.dCameraControl = dockarea.Dock("Zyla Camera Control")
        self.dMeasurement = dockarea.Dock("Measurement and saving")
        self.dConsole = dockarea.Dock("Console for debugging")
        self.dCameraScanAnal = dockarea.Dock("Zyla Camera Scan Analysis")
        
        self.area.addDock(self.dCameraView)
        self.area.addDock(self.dCameraScanAnal, "left", self.dCameraView)
        self.area.addDock(self.dCameraTimeAnal, "above", self.dCameraScanAnal)
        self.area.addDock(self.dCameraControl, "right", self.dCameraView)
        self.area.addDock(self.dConsole, "bottom", self.dCameraControl)
        self.area.addDock(self.dMeasurement, "above", self.dConsole)

        #measurement frame
        self.measurementFrame = MeasurementFrame()
        self.dMeasurement.addWidget(self.measurementFrame)
        
        # button spinbox and label
        self.cameraButton = QPushButton("Start Acquisition")
        self.isCameraAcquiring = False
        self.dCameraControl.addWidget(self.cameraButton)
        self.label = QLabel("Counter of frames")
        self.dCameraView.addWidget(self.label)

        self.param_tree = CameraParameterTree()
        self.dCameraControl.addWidget(self.param_tree.t)

        # layouts and viewboxes
        self.ImageLayout = pg.GraphicsLayoutWidget()
        self.view = self.ImageLayout.addViewBox()
        self.AnalysisLayout = pg.GraphicsLayoutWidget()

        # image layout
        self.img = pg.ImageItem(border="w")
        self.imv = pg.ImageView(imageItem=self.img, view=self.view)
        self.imv.ui.roiBtn.setText('Useless')
        self.dCameraView.addWidget(self.imv)

        # analysis layout
        self.dCameraTimeAnal.addWidget(self.AnalysisLayout)
        self.p = self.AnalysisLayout.addPlot(row=0, col=0, title="Time plot")
        self.p.setRange(QtCore.QRectF(0, 0, 50, 255))
        self.p.setAutoPan(y=True)
        # self.p.setRange(xRange = (0, 10), yRange = (0, 255))
        self.curve = self.p.plot(pen="y")

        # rOI
        self.rois = []
        self.rois.append(CustomRectangularROI([0, 0]))
        for r in self.rois:
            self.view.addItem(r)
            # r.sigRegionChanged.connect(self.updateRoi)

        self.ROIdata = np.zeros(1000)
        self.iROIdata = 0

        self.current_image_size = 2048
        self._initialize_image()

    def make_connections(self, backend):
        # backend connections
        backend.imageReadySignal.connect(self.updateImage)
        backend.measurementStartingSignal.connect(self.measurement_starting)
        backend.measurementEndingSignal.connect(self.measurement_ending)
        
        # internal connections
        self.frames_checker_timer.timeout.connect(self.frame_counter)
        self.cameraButton.clicked.connect(self.camera_button_pressed)

        self.console = console.ConsoleWidget(
            namespace = {'np': np, 'cam': backend.cam, 'roi': self.rois[0]}
        )
        self.dConsole.addWidget(self.console)
        
        self.measurementFrame.startMeasureButton.clicked.connect(self.emit_measurement_parameters)
        self.measurementFrame.folderButton.clicked.connect(self.measurementFrame.select_folder)
        self.measurementFrame.selectScanSignal.activated[str].connect(self.measurementFrame.configure_spinBoxes)   
        self.measurementFrame.startValue.valueChanged.connect(self.measurementFrame.update_tree)
        self.measurementFrame.endValue.valueChanged.connect(self.measurementFrame.update_tree)
        self.measurementFrame.stepsNum.valueChanged.connect(self.measurementFrame.update_tree)
        
        

    def camera_button_pressed(self):

        if self.isCameraAcquiring:
            self.frames_checker_timer.stop()
            self.cameraButton.setText('Start Acquisition')
            self.measurementFrame.startMeasureButton.setEnabled(True)
            self.isCameraAcquiring = False
            
        else:
            self.repositionRoi()
            self.cameraButton.setText('Stop Acquisition')
            self.measurementFrame.startMeasureButton.setEnabled(False)
            self.frame_index = 0
            self.previous_frame_index = 0
            self.frames_checker_timer.start(1000)
            self.isCameraAcquiring = True

    def _initialize_image(self):
        # lock the aspect ratio so pixels are always square
        self.view.setAspectLocked(True)
        self.view.addItem(self.img)

        self.frame_index = 0
        self.previous_frame_index = 0
        self.frames_checker_timer.start(1000)

    def updateRois(self):

        for roi in self.rois:
            roiSlice = roi.getArrayRegion(self.img.image, img=self.img)
            self.newData = roiSlice.sum()
            self.ROIdata[self.iROIdata] = self.newData
            self.iROIdata = np.mod(self.frame_index + 1, len(self.ROIdata))
            self.curve.setData(self.ROIdata)

    def repositionRoi(self):

        old_size = self.current_image_size
        aux_dict = {'1x1': 2048, '2x2': 1024, '4x4': 512, '8x8': 256}
        new_size = aux_dict[
            self.param_tree.p.child('Basic acq parameters')
            .child('Pixel Binning').value()
        ]
        print('sizes:', old_size, new_size)
        for roi in self.rois:
            x, y, sx, sy = roi.pos()[0], roi.pos()[1], roi.size()[0], roi.size()[1]
            new_x, new_y = new_size/old_size * x, new_size/old_size * y
            new_sx, new_sy = new_size/old_size * sx, new_size/old_size * sy
            roi.setPos([new_x, new_y])
            roi.setSize([new_sx, new_sy])

    @QtCore.pyqtSlot(np.ndarray, int)
    def updateImage(self, image, acq_index):

        if self.frame_index is not 0 and not self.frame_index == acq_index:
            print(
                "Dephasing in camera gui has occured: while frame index is %i, the acq index is %i"
                % (self.frame_index, acq_index)
            )

        # n = 1024
        self.img.setImage(image, autoDownsample=True)
        self.updateRois()
        self.frame_index = self.frame_index + 1
        self.current_image_size = image.shape[0]

    def frame_counter(self):
        self.counter = self.counter + 1
        self.label.setText(
            "Frames in last second: "
            + str(self.frame_index - self.previous_frame_index)
            + "    "
            + "Seconds since start: "
            + str(self.counter)
        )
        self.previous_frame_index = self.frame_index

    #@QtCore.pyqtSlot()
    def measurement_starting(self):
        self.cameraButton.setEnabled(False)
        self.measurementFrame.startMeasureButton.setEnabled(False)
        for l in LaserControlLayout._registry + LaserScanLayout._registry :
            l.freeze()
            
        print('gui meas start')
    #@QtCore.pyqtSlot()
    def measurement_ending(self):
        self.cameraButton.setEnabled(True)
        self.measurementFrame.startMeasureButton.setEnabled(True)
        for l in LaserControlLayout._registry + LaserScanLayout._registry :
            l.unfreeze()
        print('gui meas end')
    def __del__(self):
        print("chau gui")

    def emit_measurement_parameters(self):
        measure_params = dict()
        measure_params['signal_to_scan'] = self.measurementFrame.selectScanSignal.currentText()
        measure_params['start'] = self.measurementFrame.startValue.value()
        measure_params['end'] = self.measurementFrame.endValue.value()
        measure_params['steps'] = self.measurementFrame.stepsNum.value()
        measure_params['directory'] = self.measurementFrame.folderBrowser.toPlainText()
        
        self.signalWithMeasurementParameters.emit(measure_params)

###############################################################################

if __name__ == "__main__":
    app = QtGui.QApplication([])

    camera = AndorZyla(0)

    myGuiMainWindow = CameraGuiMainWindow()
    cameraWorker = CameraWorker(andor_camera=camera)

    cameraWorker.make_connections(myGuiMainWindow)
    myGuiMainWindow.make_connections(cameraWorker)

    cameraThread = QtCore.QThread()
    cameraWorker.moveToThread(cameraThread)

    cameraThread.start()

    measurementGui = MeasurementGui()
    MeasurementWorker = MeasurementWorker(camera)
    myGuiMainWindow.dMeasurement.addWidget(measurementGui)

    sys.exit(app.exec_())
