#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  4 11:10:20 2019

@author: martindrech


"""

import numpy as np
from PyQt5 import QtCore
import threading
import time
from MCDAQcontrolsignals import anal_control_signal
from resources.messages import show_warning
from resources.storage import Storage
from resources.scanFunctions import create_measurement_array, yield_array

class CameraWorker(QtCore.QObject):
    imageReadySignal = QtCore.pyqtSignal(np.ndarray, int)
    measurementStartingSignal = QtCore.pyqtSignal()
    measurementEndingSignal = QtCore.pyqtSignal()
    

    def __init__(self, andor_camera):
        super().__init__()

        self.cam = andor_camera
        self.cam.Init()
        self.cam.SensorCooling.setValue(True)
        print("camera worker initialized")

        self.ao_to_scan = None

    def make_connections(self, frontend):
        # frontend connections
        live_acq_params = frontend.param_tree.p
        frontend.cameraButton.clicked.connect(lambda: self.run(live_acq_params))
        frontend.signalWithMeasurementParameters.connect(self.simple_scan_measurement)

        # internal connections
        self.cam.helper.imageAquiredSignal.connect(self.new_image_acquired)

    def __del__(self):
        print("adios camera worker")


    def run(self, params):
        if self.cam.CameraAcquiring.getValue():
            self._stop_acquisition_loop()
            print("acquisition now stopping")

        else:
            self._configure_acq_parameters(params)
            self._start_acquisition_loop()
            self.acq_thread = threading.Thread(
                target=self.cam.live_acquisition_loop
            )
            self.acq_thread.start()
            print("acquisiting now starting")

    def _start_acquisition_loop(self):
        self.cam.live_acquisition_configure()
        self.cam.AcquisitionStart()
        print("frame rate: ", self.cam.FrameRate.getValue())
        print("Exp time: ", self.cam.ExposureTime.getValue())

    def _stop_acquisition_loop(self):
        self.acq_thread.do_run = False
        self.acq_thread.join()
        self.cam.AcquisitionStop()
        self.cam._flush()

    def _configure_acq_parameters(self, params):
        print('Acq parameters: ')
        for p in params.children()[0].children():
            print(p.name(), p.value())

        # setting info in parameter tree
        params.child('Basic acq parameters').child('Camera name').setValue(
            self.cam.CameraModel.getValue()
        )
        params.child('Basic acq parameters').child('Sensor Temperature').setValue(
            self.cam.SensorTemperature.getValue()
        )
        params.child('Basic acq parameters').child('Temperature Status').setValue(
            self.cam.TemperatureStatus.getString()
        )
        params.child('Basic acq parameters').child('Readout Time').setValue(
            self.cam.ReadoutTime.getValue()
        )
        # setting camera parameteres according to tree        
        self.cam.ExposureTime.setValue(
            params.child('Basic acq parameters').child('Exposure Time').value()
        )
        self.cam.FrameRate.setValue(self.cam.FrameRate.max())
        params.child('Basic acq parameters').child('Frame Rate').setValue(
            self.cam.FrameRate.getValue()
        )
        self.cam.AOIBinning.setString(
            params.child('Basic acq parameters').child('Pixel Binning').value()
        )
        self.cam.PixelReadoutRate.setString(
            params.child('Basic acq parameters').child('Pixel Readout Rate').value()
        )
        self.cam.SimplePreAmpGainControl.setString(
            params.child('Basic acq parameters').child('Dynamic Range').value()
        )        
        self.cam.SpuriousNoiseFilter.setValue(
            params.child('Basic acq parameters').child('Spurious Noise Filter').value()
        )     
        
    @QtCore.pyqtSlot(int)
    def new_image_acquired(self, acq_index):
        self.imageReadySignal.emit(self.cam.acq_queue.get(), acq_index)
        
    def simple_scan_measurement(self, measure_params):

        if self.cam.CameraAcquiring.getValue():
            show_warning('Can not start measurement if camera is acquiring')
            return 0
        
        self.measurementStartingSignal.emit()  
          
        signal_to_scan = self.get_scan_signal(measure_params['signal_to_scan'])
        start = measure_params['start']
        end = measure_params['end']
        step_num = measure_params['steps']
        directory = measure_params['directory']
        scan_array = create_measurement_array(start, end, step_num)
        storage = Storage(directory)
        
        self.cam.TriggerMode.setString('Software')
        
        def get_scan_signal():
            pass

        QtCore.QTimer.singleShot(5000, self.measurementEndingSignal.emit)
        print('Ending measurement')
        
        
        """
        scan_array = create_measurement_array()  # aca necesito parametros de los spinboxes de la measurementGUi
        prepare_camara_to_triggered_acq() #includes stopping acquisition if it is in acq live
        create_temporary_data_files
        measurement loop:
                set_scan_signal
                wait_predefined_time
                take_and_recibe_image
                process_image_to_get_data
                save_data
                update_camera_gui_with_image_and_data
                update_measurement_gui
        """

    


